/**
 *******************************************************************************
 * @file  wdt.c
 * @brief Watchdog timer control program for R-IN32M3
 * 
 * @note 
 * Copyright (C) 2012,2013 Renesas Electronics Corporation 
 * Copyright (C) 2012,2013 Renesas Micro Systems Co., Ltd. 
 * 
 * @par
 *  This is a sample program. 
 *  Renesas Electronics assumes no responsibility for any losses incurred. 
 * 
 *******************************************************************************
 */

/*============================================================================*/
/* I N C L U D E                                                              */
/*============================================================================*/
#include "wdt/wdt.h"

/*============================================================================*/
/* D E F I N E                                                                */
/*============================================================================*/

/*============================================================================*/
/* V A R I A B L E                                                            */
/*============================================================================*/

/*============================================================================*/
/* E X T E R N                                                                */
/*============================================================================*/

/*============================================================================*/
/* S T A T I C   F U N C T I O N   P R O T O T Y P E                          */
/*============================================================================*/

/*============================================================================*/
/* P R O G R A M                                                              */
/*============================================================================*/

/**
 ******************************************************************************
  @brief  Initialize
  @param  --
  @return Error condition
  @retval ER_OK -- Successful
 ******************************************************************************
 */
ER_RET wdt_init(void)
{
	/*
	 *--------------------------------------------------------------------------
	 *
	 *  System register
	 *
	 *--------------------------------------------------------------------------
	 */
	RIN_SYS->SYSPCMD = 0x000000a5;			/* sequence to release protect */
	RIN_SYS->SYSPCMD = 0x00000001;			/* sequence to release protect */
	RIN_SYS->SYSPCMD = 0x0000fffe;			/* sequence to release protect */
	RIN_SYS->SYSPCMD = 0x00000001;			/* release protect */
	{	/* Software reset */
		volatile int i;

		/* SFTRES1.RSWDT=0(active) */
		RIN_SYS->SFTRES1 = 0x0000ffef;		/* Always must be '1' except RSWDT bit */
		/* !!! Reset width should keep at least the 2 cycle width of WDT count clock !!!
		 * ( CPU clock : WDT count clock (default) = 1 : 32 )
		 */
		i = (32 * 2);
		while (i--)
		{
			__NOP();
		}
		/* SFTRES1.RSWDT=1(inactive) */
		RIN_SYS->SFTRES1 = 0x0000ffff;		/* Always must be '1' except RSWDT bit */
	}

	{	/* WDT count clock */
		RIN_SYS->WDTCLKCFG = 0x00000006;	/* [3:0]: frequency dividing rate -> HCLK/2048 */
	}

	RIN_SYS->SYSPCMD = 0x00000000;			/* set protect */

	/*
	 *--------------------------------------------------------------------------
	 *
	 *  WDT register
	 *
	 *--------------------------------------------------------------------------
	 */
	RIN_WDT->WDTAnMD =	  (0 << 7)			/* [7]  : reserve */
						| (7 << 4)			/* [6:4]: (WDTAnOVF2-0) Overflow interval time -> (count clk)/2^16 */
						| (1 << 2)			/* [2]  : (WDTAnERM   ) Error mode -> Reset mode */
						| (3 << 0);			/* [1:0]: (WDTAnWS1-0 ) Window size -> 100% */

	return ER_OK;
}

/**
 ******************************************************************************
  @brief  Start
  @param  --
  @return Error condition
  @retval ER_OK -- Successful
 ******************************************************************************
 */
ER_RET wdt_start(void)
{
	RIN_WDT->WDTAnWDTE = 0xAC;				/* [7]  : (WDTAnRUN   ) Watchdog timer start */

	return ER_OK;
}

/**
 ******************************************************************************
  @brief  Clear
  @param  --
  @return Error condition
  @retval ER_OK -- Successful
 ******************************************************************************
 */
ER_RET wdt_clear(void)
{
	RIN_WDT->WDTAnWDTE = 0xAC;				/* to clear the counter */

	return ER_OK;
}

/**
 ******************************************************************************
  @brief  Wait reset
  @param  --
  @return none
 ******************************************************************************
 */
void wdt_wait_reset(void)
{
	/* disable interrupt */
	__disable_irq();

	/* wait reset */
	while (1)
	{
		__NOP();
	}
}

